@@ -4,7 +4,7 @@
  * \brief    Post-processing to generate .mp4 files (headers)
  * \details  Implementation of the post-processing code (based on FFmpeg)
  * needed to generate .mp4 files out of H.264 RTP frames.
- * 
+ *
  * \ingroup postprocessing
  * \ref postprocessing
  */
@@ -17,7 +17,7 @@
 #include "pp-rtp.h"
 
 /* H.264 stuff */
-int janus_pp_h264_create(char *destination);
+int janus_pp_h264_create(char *destination, char *metadata);
 int janus_pp_h264_preprocess(FILE *file, janus_pp_frame_packet *list);
 int janus_pp_h264_process(FILE *file, janus_pp_frame_packet *list, int *working);
 void janus_pp_h264_close(void);
