@@ -7,13 +7,13 @@
  * of such object by playing with its properties, and a serialization
  * to an SDP string that can be passed around. Since they don't have any
  * core dependencies, these utilities can be used by plugins as well.
- * 
+ *
  * \ingroup core
  * \ref core
  */
 
 #include <string.h>
- 
+
 #include "sdp-utils.h"
 #include "utils.h"
 #include "debug.h"
@@ -810,14 +810,14 @@ char *janus_sdp_write(janus_sdp *imported) {
 			g_snprintf(buffer, sizeof(buffer), "a=%s\r\n", direction);
 			g_strlcat(sdp, buffer, JANUS_BUFSIZE);
 		}
-		if(m->port == 0) {
-			/* No point going on */
-			temp = temp->next;
-			continue;
-		}
 		GList *temp2 = m->attributes;
 		while(temp2) {
 			janus_sdp_attribute *a = (janus_sdp_attribute *)temp2->data;
+			if(m->port == 0 && strcasecmp(a->name, "mid")) {
+				/* This media has been rejected or disabled: we only add the mid attribute, if available */
+				temp2 = temp2->next;
+				continue;
+			}
 			if(a->value != NULL) {
 				g_snprintf(buffer, sizeof(buffer), "a=%s:%s\r\n", a->name, a->value);
 			} else {
@@ -956,7 +956,8 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 	va_start(args, address);
 	/* Let's see what we should do with the media */
 	gboolean do_audio = TRUE, do_video = TRUE, do_data = TRUE,
-		audio_dtmf = FALSE, video_rtcpfb = TRUE, h264_fmtp = TRUE;
+		audio_dtmf = FALSE, video_rtcpfb = TRUE, h264_fmtp = TRUE,
+		data_legacy = TRUE;
 	const char *audio_codec = NULL, *video_codec = NULL;
 	int audio_pt = 111, video_pt = 96;
 	janus_sdp_mdirection audio_dir = JANUS_SDP_SENDRECV, video_dir = JANUS_SDP_SENDRECV;
@@ -986,6 +987,8 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 			video_rtcpfb = va_arg(args, gboolean);
 		} else if(property == JANUS_SDP_OA_VIDEO_H264_FMTP) {
 			h264_fmtp = va_arg(args, gboolean);
+		} else if(property == JANUS_SDP_OA_DATA_LEGACY) {
+			data_legacy = va_arg(args, gboolean);
 		} else {
 			JANUS_LOG(LOG_WARN, "Unknown property %d for preparing SDP answer, ignoring...\n", property);
 		}
@@ -1059,13 +1062,19 @@ janus_sdp *janus_sdp_generate_offer(const char *name, const char *address, ...)
 		offer->m_lines = g_list_append(offer->m_lines, m);
 	}
 	if(do_data) {
-		janus_sdp_mline *m = janus_sdp_mline_create(JANUS_SDP_APPLICATION, 1, "DTLS/SCTP", JANUS_SDP_DEFAULT);
+		janus_sdp_mline *m = janus_sdp_mline_create(JANUS_SDP_APPLICATION, 1,
+			data_legacy ? "DTLS/SCTP" : "UDP/DTLS/SCTP", JANUS_SDP_DEFAULT);
 		m->c_ipv4 = TRUE;
 		m->c_addr = g_strdup(offer->c_addr);
-		m->fmts = g_list_append(m->fmts, g_strdup("5000"));
-		/* Add an sctmap attribute */
-		janus_sdp_attribute *aa = janus_sdp_attribute_create("sctmap", "5000 webrtc-datachannel 16");
-		m->attributes = g_list_append(m->attributes, aa);
+		m->fmts = g_list_append(m->fmts, g_strdup(data_legacy ? "5000" : "webrtc-datachannel"));
+		/* Add an sctpmap attribute */
+		if(data_legacy) {
+			janus_sdp_attribute *aa = janus_sdp_attribute_create("sctpmap", "5000 webrtc-datachannel 16");
+			m->attributes = g_list_append(m->attributes, aa);
+		} else {
+			janus_sdp_attribute *aa = janus_sdp_attribute_create("sctp-port", "5000");
+			m->attributes = g_list_append(m->attributes, aa);
+		}
 		offer->m_lines = g_list_append(offer->m_lines, m);
 	}
 
@@ -1246,7 +1255,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 							codec = "pcma";
 							if(janus_sdp_get_codec_pt(offer, codec) < 0) {
 								/* a-law not found, maybe G.722? */
-								codec = "722";
+								codec = "g722";
 								if(janus_sdp_get_codec_pt(offer, codec) < 0) {
 									/* G.722 not found, maybe isac32? */
 									codec = "isac32";
@@ -1318,7 +1327,7 @@ janus_sdp *janus_sdp_generate_answer(janus_sdp *offer, ...) {
 				}
 			}
 		} else {
-			/* This is for data, add formats and an sctmap attribute */
+			/* This is for data, add formats and an sctpmap attribute */
 			am->direction = JANUS_SDP_DEFAULT;
 			GList *fmt = m->fmts;
 			while(fmt) {
