@@ -6,7 +6,7 @@
  * Janus API, using the libwebsockets library (http://libwebsockets.org).
  * This means that, with the help of this module, browsers or applications
  * (e.g., nodejs server side implementations) can also make use of
- * WebSockets to make requests to the gateway. In that case, the same
+ * WebSockets to make requests to Janus. In that case, the same
  * WebSocket can be used for both sending requests and receiving
  * notifications, without the need for long polls. At the same time,
  * without the concept of a REST path, requests sent through the
@@ -111,6 +111,42 @@ static size_t json_format = JSON_INDENT(3) | JSON_PRESERVE_ORDER;
 
 /* Logging */
 static int ws_log_level = 0;
+static const char *janus_websockets_get_level_str(int level) {
+	switch(level) {
+		case LLL_ERR:
+			return "ERR";
+		case LLL_WARN:
+			return "WARN";
+		case LLL_NOTICE:
+			return "NOTICE";
+		case LLL_INFO:
+			return "INFO";
+		case LLL_DEBUG:
+			return "DEBUG";
+		case LLL_PARSER:
+			return "PARSER";
+		case LLL_HEADER:
+			return "HEADER";
+		case LLL_EXT:
+			return "EXT";
+		case LLL_CLIENT:
+			return "CLIENT";
+		case LLL_LATENCY:
+			return "LATENCY";
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 2) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
+		case LLL_USER:
+			return "USER";
+#endif
+		case LLL_COUNT:
+			return "COUNT";
+		default:
+			return NULL;
+	}
+}
+static void janus_websockets_log_emit_function(int level, const char *line) {
+	/* FIXME Do we want to use different Janus debug levels according to the level here? */
+	JANUS_LOG(LOG_INFO, "[libwebsockets][%s] %s", janus_websockets_get_level_str(level), line);
+}
 
 /* WebSockets service thread */
 static GThread *ws_thread = NULL;
@@ -232,21 +268,28 @@ static const char *janus_websockets_reason_string(enum lws_callback_reasons reas
 /* Helper method to return the interface associated with a local IP address */
 static char *janus_websockets_get_interface_name(const char *ip) {
 	struct ifaddrs *addrs = NULL, *iap = NULL;
-	getifaddrs(&addrs);
+	if(getifaddrs(&addrs) == -1)
+		return NULL;
 	for(iap = addrs; iap != NULL; iap = iap->ifa_next) {
 		if(iap->ifa_addr && (iap->ifa_flags & IFF_UP)) {
 			if(iap->ifa_addr->sa_family == AF_INET) {
 				struct sockaddr_in *sa = (struct sockaddr_in *)(iap->ifa_addr);
 				char buffer[16];
 				inet_ntop(iap->ifa_addr->sa_family, (void *)&(sa->sin_addr), buffer, sizeof(buffer));
-				if(!strcmp(ip, buffer))
-					return g_strdup(iap->ifa_name);
+				if(!strcmp(ip, buffer)) {
+					char *iface = g_strdup(iap->ifa_name);
+					freeifaddrs(addrs);
+					return iface;
+				}
 			} else if(iap->ifa_addr->sa_family == AF_INET6) {
 				struct sockaddr_in6 *sa = (struct sockaddr_in6 *)(iap->ifa_addr);
 				char buffer[48];
 				inet_ntop(iap->ifa_addr->sa_family, (void *)&(sa->sin6_addr), buffer, sizeof(buffer));
-				if(!strcmp(ip, buffer))
-					return g_strdup(iap->ifa_name);
+				if(!strcmp(ip, buffer)) {
+					char *iface = g_strdup(iap->ifa_name);
+					freeifaddrs(addrs);
+					return iface;
+				}
 			}
 		}
 	}
@@ -310,7 +353,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 	JANUS_LOG(LOG_WARN, "libwebsockets has been built without IPv6 support, will bind to IPv4 only\n");
 #endif
 
-	/* This is the callback we'll need to invoke to contact the gateway */
+	/* This is the callback we'll need to invoke to contact the Janus core */
 	gateway = callback;
 
 	/* Prepare the common context */
@@ -324,14 +367,23 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 
 	/* Read configuration */
 	char filename[255];
-	g_snprintf(filename, 255, "%s/%s.cfg", config_path, JANUS_WEBSOCKETS_PACKAGE);
+	g_snprintf(filename, 255, "%s/%s.jcfg", config_path, JANUS_WEBSOCKETS_PACKAGE);
 	JANUS_LOG(LOG_VERB, "Configuration file: %s\n", filename);
 	janus_config *config = janus_config_parse(filename);
+	if(config == NULL) {
+		JANUS_LOG(LOG_WARN, "Couldn't find .jcfg configuration file (%s), trying .cfg\n", JANUS_WEBSOCKETS_PACKAGE);
+		g_snprintf(filename, 255, "%s/%s.cfg", config_path, JANUS_WEBSOCKETS_PACKAGE);
+		JANUS_LOG(LOG_VERB, "Configuration file: %s\n", filename);
+		config = janus_config_parse(filename);
+	}
 	if(config != NULL) {
 		janus_config_print(config);
+		janus_config_category *config_general = janus_config_get_create(config, NULL, janus_config_type_category, "general");
+		janus_config_category *config_admin = janus_config_get_create(config, NULL, janus_config_type_category, "admin");
+		janus_config_category *config_certs = janus_config_get_create(config, NULL, janus_config_type_category, "certificates");
 
 		/* Handle configuration */
-		janus_config_item *item = janus_config_get_item_drilldown(config, "general", "json");
+		janus_config_item *item = janus_config_get(config, config_general, janus_config_type_item, "json");
 		if(item && item->value) {
 			/* Check how we need to format/serialize the JSON output */
 			if(!strcasecmp(item->value, "indented")) {
@@ -350,24 +402,63 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		}
 
 		/* Check if we need to send events to handlers */
-		janus_config_item *events = janus_config_get_item_drilldown(config, "general", "events");
+		janus_config_item *events = janus_config_get(config, config_general, janus_config_type_item, "events");
 		if(events != NULL && events->value != NULL)
 			notify_events = janus_is_true(events->value);
 		if(!notify_events && callback->events_is_enabled()) {
 			JANUS_LOG(LOG_WARN, "Notification of events to handlers disabled for %s\n", JANUS_WEBSOCKETS_NAME);
 		}
 
-		item = janus_config_get_item_drilldown(config, "general", "ws_logging");
+		item = janus_config_get(config, config_general, janus_config_type_item, "ws_logging");
 		if(item && item->value) {
-			ws_log_level = atoi(item->value);
-			if(ws_log_level < 0)
-				ws_log_level = 0;
+			/* libwebsockets uses a mask to set log levels, as documented here:
+			 * https://libwebsockets.org/lws-api-doc-master/html/group__log.html */
+			if(strstr(item->value, "none")) {
+				/* Disable libwebsockets logging completely (the default) */
+			} else if(strstr(item->value, "all")) {
+				/* Enable all libwebsockets logging */
+				ws_log_level = LLL_ERR | LLL_WARN | LLL_NOTICE | LLL_INFO |
+					LLL_DEBUG | LLL_PARSER | LLL_HEADER | LLL_EXT |
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 2) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
+					LLL_CLIENT | LLL_LATENCY | LLL_USER | LLL_COUNT;
+#else
+					LLL_CLIENT | LLL_LATENCY | LLL_COUNT;
+#endif
+			} else {
+				/* Only enable some of the properties */
+				if(strstr(item->value, "err"))
+					ws_log_level |= LLL_ERR;
+				if(strstr(item->value, "warn"))
+					ws_log_level |= LLL_WARN;
+				if(strstr(item->value, "notice"))
+					ws_log_level |= LLL_NOTICE;
+				if(strstr(item->value, "info"))
+					ws_log_level |= LLL_INFO;
+				if(strstr(item->value, "debug"))
+					ws_log_level |= LLL_DEBUG;
+				if(strstr(item->value, "parser"))
+					ws_log_level |= LLL_PARSER;
+				if(strstr(item->value, "header"))
+					ws_log_level |= LLL_HEADER;
+				if(strstr(item->value, "ext"))
+					ws_log_level |= LLL_EXT;
+				if(strstr(item->value, "client"))
+					ws_log_level |= LLL_CLIENT;
+				if(strstr(item->value, "latency"))
+					ws_log_level |= LLL_LATENCY;
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 2) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
+				if(strstr(item->value, "user"))
+					ws_log_level |= LLL_USER;
+#endif
+				if(strstr(item->value, "count"))
+					ws_log_level |= LLL_COUNT;
+			}
 		}
-		JANUS_LOG(LOG_VERB, "libwebsockets logging: %d\n", ws_log_level);
-		lws_set_log_level(ws_log_level, NULL);
+		JANUS_LOG(LOG_INFO, "libwebsockets logging: %d\n", ws_log_level);
+		lws_set_log_level(ws_log_level, janus_websockets_log_emit_function);
 
 		/* Any ACL for either the Janus or Admin API? */
-		item = janus_config_get_item_drilldown(config, "general", "ws_acl");
+		item = janus_config_get(config, config_general, janus_config_type_item, "ws_acl");
 		if(item && item->value) {
 			gchar **list = g_strsplit(item->value, ",", -1);
 			gchar *index = list[0];
@@ -385,7 +476,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			g_strfreev(list);
 			list = NULL;
 		}
-		item = janus_config_get_item_drilldown(config, "admin", "admin_ws_acl");
+		item = janus_config_get(config, config_admin, janus_config_type_item, "admin_ws_acl");
 		if(item && item->value) {
 			gchar **list = g_strsplit(item->value, ",", -1);
 			gchar *index = list[0];
@@ -406,9 +497,9 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 
 		/* Check if we need to enable the transport level ping/pong mechanism */
 		int pingpong_trigger = 0, pingpong_timeout = 0;
-		item = janus_config_get_item_drilldown(config, "general", "pingpong_trigger");
+		item = janus_config_get(config, config_general, janus_config_type_item, "pingpong_trigger");
 		if(item && item->value) {
-#if LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
 			pingpong_trigger = atoi(item->value);
 			if(pingpong_trigger < 0) {
 				JANUS_LOG(LOG_WARN, "Invalid value for pingpong_trigger (%d), ignoring...\n", pingpong_trigger);
@@ -418,9 +509,9 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			JANUS_LOG(LOG_WARN, "WebSockets ping/pong only supported in libwebsockets >= 2.1\n");
 #endif
 		}
-		item = janus_config_get_item_drilldown(config, "general", "pingpong_timeout");
+		item = janus_config_get(config, config_general, janus_config_type_item, "pingpong_timeout");
 		if(item && item->value) {
-#if LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
 			pingpong_timeout = atoi(item->value);
 			if(pingpong_timeout < 0) {
 				JANUS_LOG(LOG_WARN, "Invalid value for pingpong_timeout (%d), ignoring...\n", pingpong_timeout);
@@ -433,7 +524,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		if((pingpong_trigger && !pingpong_timeout) || (!pingpong_trigger && pingpong_timeout)) {
 			JANUS_LOG(LOG_WARN, "pingpong_trigger and pingpong_timeout not both set, ignoring...\n");
 		}
-#if LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
 		if(pingpong_trigger > 0 && pingpong_timeout > 0) {
 			wscinfo.ws_ping_pong_interval = pingpong_trigger;
 			wscinfo.timeout_secs = pingpong_timeout;
@@ -451,20 +542,20 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		}
 
 		/* Setup the Janus API WebSockets server(s) */
-		item = janus_config_get_item_drilldown(config, "general", "ws");
+		item = janus_config_get(config, config_general, janus_config_type_item, "ws");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "WebSockets server disabled\n");
 		} else {
 			int wsport = 8188;
-			item = janus_config_get_item_drilldown(config, "general", "ws_port");
+			item = janus_config_get(config, config_general, janus_config_type_item, "ws_port");
 			if(item && item->value)
 				wsport = atoi(item->value);
 			char *interface = NULL;
-			item = janus_config_get_item_drilldown(config, "general", "ws_interface");
+			item = janus_config_get(config, config_general, janus_config_type_item, "ws_interface");
 			if(item && item->value)
 				interface = (char *)item->value;
 			char *ip = NULL;
-			item = janus_config_get_item_drilldown(config, "general", "ws_ip");
+			item = janus_config_get(config, config_general, janus_config_type_item, "ws_ip");
 			if(item && item->value) {
 				ip = (char *)item->value;
 				char *iface = janus_websockets_get_interface_name(ip);
@@ -495,20 +586,20 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			}
 			g_free(ip);
 		}
-		item = janus_config_get_item_drilldown(config, "general", "wss");
+		item = janus_config_get(config, config_general, janus_config_type_item, "wss");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Secure WebSockets server disabled\n");
 		} else {
 			int wsport = 8989;
-			item = janus_config_get_item_drilldown(config, "general", "wss_port");
+			item = janus_config_get(config, config_general, janus_config_type_item, "wss_port");
 			if(item && item->value)
 				wsport = atoi(item->value);
 			char *interface = NULL;
-			item = janus_config_get_item_drilldown(config, "general", "wss_interface");
+			item = janus_config_get(config, config_general, janus_config_type_item, "wss_interface");
 			if(item && item->value)
 				interface = (char *)item->value;
 			char *ip = NULL;
-			item = janus_config_get_item_drilldown(config, "general", "wss_ip");
+			item = janus_config_get(config, config_general, janus_config_type_item, "wss_ip");
 			if(item && item->value) {
 				ip = (char *)item->value;
 				char *iface = janus_websockets_get_interface_name(ip);
@@ -517,17 +608,17 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				}
 				ip = iface;
 			}
-			item = janus_config_get_item_drilldown(config, "certificates", "cert_pem");
+			item = janus_config_get(config, config_certs, janus_config_type_item, "cert_pem");
 			if(!item || !item->value) {
 				JANUS_LOG(LOG_FATAL, "Missing certificate/key path\n");
 			} else {
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
 				char *password = NULL;
-				item = janus_config_get_item_drilldown(config, "certificates", "cert_key");
+				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
-				item = janus_config_get_item_drilldown(config, "certificates", "cert_pwd");
+				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_pwd");
 				if(item && item->value)
 					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
@@ -555,24 +646,24 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				} else {
 					JANUS_LOG(LOG_INFO, "Secure WebSockets server started (port %d)...\n", wsport);
 				}
-				g_free(ip);
 			}
+			g_free(ip);
 		}
 		/* Do the same for the Admin API, if enabled */
-		item = janus_config_get_item_drilldown(config, "admin", "admin_ws");
+		item = janus_config_get(config, config_admin, janus_config_type_item, "admin_ws");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Admin WebSockets server disabled\n");
 		} else {
 			int wsport = 7188;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_ws_port");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_ws_port");
 			if(item && item->value)
 				wsport = atoi(item->value);
 			char *interface = NULL;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_ws_interface");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_ws_interface");
 			if(item && item->value)
 				interface = (char *)item->value;
 			char *ip = NULL;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_ws_ip");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_ws_ip");
 			if(item && item->value) {
 				ip = (char *)item->value;
 				char *iface = janus_websockets_get_interface_name(ip);
@@ -603,20 +694,20 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 			}
 			g_free(ip);
 		}
-		item = janus_config_get_item_drilldown(config, "admin", "admin_wss");
+		item = janus_config_get(config, config_admin, janus_config_type_item, "admin_wss");
 		if(!item || !item->value || !janus_is_true(item->value)) {
 			JANUS_LOG(LOG_WARN, "Secure Admin WebSockets server disabled\n");
 		} else {
 			int wsport = 7989;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_wss_port");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_wss_port");
 			if(item && item->value)
 				wsport = atoi(item->value);
 			char *interface = NULL;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_wss_interface");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_wss_interface");
 			if(item && item->value)
 				interface = (char *)item->value;
 			char *ip = NULL;
-			item = janus_config_get_item_drilldown(config, "admin", "admin_wss_ip");
+			item = janus_config_get(config, config_admin, janus_config_type_item, "admin_wss_ip");
 			if(item && item->value) {
 				ip = (char *)item->value;
 				char *iface = janus_websockets_get_interface_name(ip);
@@ -625,17 +716,17 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				}
 				ip = iface;
 			}
-			item = janus_config_get_item_drilldown(config, "certificates", "cert_pem");
+			item = janus_config_get(config, config_certs, janus_config_type_item, "cert_pem");
 			if(!item || !item->value) {
 				JANUS_LOG(LOG_FATAL, "Missing certificate/key path\n");
 			} else {
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
 				char *password = NULL;
-				item = janus_config_get_item_drilldown(config, "certificates", "cert_key");
+				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
-				item = janus_config_get_item_drilldown(config, "certificates", "cert_pwd");
+				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_pwd");
 				if(item && item->value)
 					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
@@ -663,8 +754,8 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				} else {
 					JANUS_LOG(LOG_INFO, "Secure Admin WebSockets server started (port %d)...\n", wsport);
 				}
-				g_free(ip);
 			}
+			g_free(ip);
 		}
 	}
 	janus_config_destroy(config);
@@ -733,8 +824,6 @@ static void janus_websockets_destroy_client(
 		json_object_set_new(info, "event", json_string("disconnected"));
 		gateway->notify_event(&janus_websockets_transport, ws_client->ts, info);
 	}
-	/* Notify core */
-	gateway->transport_gone(&janus_websockets_transport, ws_client->ts);
 	ws_client->ts->transport_p = NULL;
 	/* Remove messages queue too, if needed */
 	if(ws_client->messages != NULL) {
@@ -753,6 +842,8 @@ static void janus_websockets_destroy_client(
 	ws_client->bufpending = 0;
 	ws_client->bufoffset = 0;
 	janus_mutex_unlock(&ws_client->ts->mutex);
+	/* Notify core */
+	gateway->transport_gone(&janus_websockets_transport, ws_client->ts);
 	janus_transport_session_destroy(ws_client->ts);
 }
 
