@@ -4,7 +4,7 @@
  * \brief    Utilities and helpers
  * \details  Implementations of a few methods that may be of use here
  * and there in the code.
- * 
+ *
  * \ingroup core
  * \ref core
  */
@@ -366,7 +366,7 @@ int janus_pidfile_create(const char *file) {
 		return 0;
 	pidfile = g_strdup(file);
 	/* Try creating a PID file (or opening an existing one) */
-	pidfd = open(pidfile, O_RDWR|O_CREAT, 0644);
+	pidfd = open(pidfile, O_RDWR|O_CREAT|O_TRUNC, 0644);
 	if(pidfd < 0) {
 		JANUS_LOG(LOG_FATAL, "Error opening/creating PID file %s, does Janus have enough permissions?\n", pidfile);
 		return -1;
@@ -496,8 +496,8 @@ gboolean janus_json_is_valid(json_t *val, json_type jtype, unsigned int flags) {
 	# define swap2(d) d
 #endif
 
-gboolean janus_vp8_is_keyframe(char* buffer, int len) {
-	if(!buffer || len < 0)
+gboolean janus_vp8_is_keyframe(const char *buffer, int len) {
+	if(!buffer || len < 16)
 		return FALSE;
 	/* Parse VP8 header now */
 	uint8_t vp8pd = *buffer;
@@ -555,10 +555,13 @@ gboolean janus_vp8_is_keyframe(char* buffer, int len) {
 				if(c[0]!=0x9d||c[1]!=0x01||c[2]!=0x2a) {
 					JANUS_LOG(LOG_HUGE, "First 3-bytes after header not what they're supposed to be?\n");
 				} else {
-					int vp8w = swap2(*(unsigned short*)(c+3))&0x3fff;
-					int vp8ws = swap2(*(unsigned short*)(c+3))>>14;
-					int vp8h = swap2(*(unsigned short*)(c+5))&0x3fff;
-					int vp8hs = swap2(*(unsigned short*)(c+5))>>14;
+					unsigned short val3, val5;
+					memcpy(&val3,c+3,sizeof(short));
+					int vp8w = swap2(val3)&0x3fff;
+					int vp8ws = swap2(val3)>>14;
+					memcpy(&val5,c+5,sizeof(short));
+					int vp8h = swap2(val5)&0x3fff;
+					int vp8hs = swap2(val5)>>14;
 					JANUS_LOG(LOG_HUGE, "Got a VP8 key frame: %dx%d (scale=%dx%d)\n", vp8w, vp8h, vp8ws, vp8hs);
 					return TRUE;
 				}
@@ -569,8 +572,8 @@ gboolean janus_vp8_is_keyframe(char* buffer, int len) {
 	return FALSE;
 }
 
-gboolean janus_vp9_is_keyframe(char* buffer, int len) {
-	if(!buffer || len < 0)
+gboolean janus_vp9_is_keyframe(const char *buffer, int len) {
+	if(!buffer || len < 16)
 		return FALSE;
 	/* Parse VP9 header now */
 	uint8_t vp9pd = *buffer;
@@ -580,6 +583,7 @@ gboolean janus_vp9_is_keyframe(char* buffer, int len) {
 	uint8_t fbit = (vp9pd & 0x10);
 	uint8_t vbit = (vp9pd & 0x02);
 	buffer++;
+	len--;
 	if(ibit) {
 		/* Read the PictureID octet */
 		vp9pd = *buffer;
@@ -587,18 +591,22 @@ gboolean janus_vp9_is_keyframe(char* buffer, int len) {
 		uint8_t mbit = (vp9pd & 0x80);
 		if(!mbit) {
 			buffer++;
+			len--;
 		} else {
 			memcpy(&picid, buffer, sizeof(uint16_t));
 			wholepicid = ntohs(picid);
 			picid = (wholepicid & 0x7FFF);
 			buffer += 2;
+			len -= 2;
 		}
 	}
 	if(lbit) {
 		buffer++;
+		len--;
 		if(!fbit) {
 			/* Non-flexible mode, skip TL0PICIDX */
 			buffer++;
+			len--;
 		}
 	}
 	if(fbit && pbit) {
@@ -608,6 +616,9 @@ gboolean janus_vp9_is_keyframe(char* buffer, int len) {
 			vp9pd = *buffer;
 			nbit = (vp9pd & 0x01);
 			buffer++;
+			len--;
+			if(len == 0)	/* Make sure we don't overflow */
+				return FALSE;
 		}
 	}
 	if(vbit) {
@@ -619,15 +630,20 @@ gboolean janus_vp9_is_keyframe(char* buffer, int len) {
 		if(ybit) {
 			/* Iterate on all spatial layers and get resolution */
 			buffer++;
+			len--;
+			if(len == 0)	/* Make sure we don't overflow */
+				return FALSE;
 			uint i=0;
-			for(i=0; i<n_s; i++) {
+			for(i=0; i<n_s && len>=4; i++,len-=4) {
 				/* Width */
-				uint16_t *w = (uint16_t *)buffer;
-				int vp9w = ntohs(*w);
+				uint16_t w;
+				memcpy(&w, buffer, sizeof(uint16_t));
+				int vp9w = ntohs(w);
 				buffer += 2;
 				/* Height */
-				uint16_t *h = (uint16_t *)buffer;
-				int vp9h = ntohs(*h);
+				uint16_t h;
+				memcpy(&h, buffer, sizeof(uint16_t));
+				int vp9h = ntohs(h);
 				buffer += 2;
 				if(vp9w || vp9h) {
 					JANUS_LOG(LOG_HUGE, "Got a VP9 key frame: %dx%d\n", vp9w, vp9h);
@@ -640,17 +656,36 @@ gboolean janus_vp9_is_keyframe(char* buffer, int len) {
 	return FALSE;
 }
 
-gboolean janus_h264_is_keyframe(char* buffer, int len) {
-	if(!buffer || len < 0)
+gboolean janus_h264_is_keyframe(const char *buffer, int len) {
+	if(!buffer || len < 16)
 		return FALSE;
 	/* Parse H264 header now */
 	uint8_t fragment = *buffer & 0x1F;
 	uint8_t nal = *(buffer+1) & 0x1F;
 	uint8_t start_bit = *(buffer+1) & 0x80;
 	if(fragment == 5 ||
-			((fragment == 28 || fragment == 29) && nal == 5 && start_bit == 128)) {
+			((fragment == 28 || fragment == 29) && (nal == 5 || nal == 7) && start_bit == 128)) {
 		JANUS_LOG(LOG_HUGE, "Got an H264 key frame\n");
 		return TRUE;
+	} else if(fragment == 24) {
+		/* May we find an SPS in this STAP-A? */
+		buffer++;
+		len--;
+		uint16_t psize = 0;
+		/* We're reading 3 bytes */
+		while(len > 2) {
+			memcpy(&psize, buffer, 2);
+			psize = ntohs(psize);
+			buffer += 2;
+			len -= 2;
+			int nal = *buffer & 0x1F;
+			if(nal == 7) {
+				JANUS_LOG(LOG_HUGE, "Got an SPS/PPS\n");
+				return TRUE;
+			}
+			buffer += psize;
+			len -= psize;
+		}
 	}
 	/* If we got here it's not a key frame */
 	return FALSE;
@@ -658,7 +693,7 @@ gboolean janus_h264_is_keyframe(char* buffer, int len) {
 
 int janus_vp8_parse_descriptor(char *buffer, int len,
 		uint16_t *picid, uint8_t *tl0picidx, uint8_t *tid, uint8_t *y, uint8_t *keyidx) {
-	if(!buffer || len < 0)
+	if(!buffer || len < 6)
 		return -1;
 	if(picid)
 		*picid = 0;
@@ -718,7 +753,7 @@ int janus_vp8_parse_descriptor(char *buffer, int len,
 }
 
 static int janus_vp8_replace_descriptor(char *buffer, int len, uint16_t picid, uint8_t tl0picidx) {
-	if(!buffer || len < 0)
+	if(!buffer || len < 6)
 		return -1;
 	uint8_t vp8pd = *buffer;
 	uint8_t xbit = (vp8pd & 0x80);
@@ -797,7 +832,7 @@ void janus_vp8_simulcast_descriptor_update(char *buffer, int len, janus_vp8_simu
 int janus_vp9_parse_svc(char *buffer, int len, int *found,
 		int *spatial_layer, int *temporal_layer,
 		uint8_t *p, uint8_t *d, uint8_t *u, uint8_t *b, uint8_t *e) {
-	if(!buffer || len < 0)
+	if(!buffer || len < 8)
 		return -1;
 	/* VP9 depay: */
 		/* https://tools.ietf.org/html/draft-ietf-payload-vp9-04 */
@@ -877,6 +912,8 @@ int janus_vp9_parse_svc(char *buffer, int len, int *found,
 			nbit = (vp9pd & 0x01);
 			buffer++;
 			len--;
+			if(len == 0)	/* Make sure we don't overflow */
+				return -1;
 		}
 	}
 	if(vbit) {
@@ -891,22 +928,30 @@ int janus_vp9_parse_svc(char *buffer, int len, int *found,
 			/* Iterate on all spatial layers and get resolution */
 			buffer++;
 			len--;
+			if(len == 0)	/* Make sure we don't overflow */
+				return -1;
 			int i=0;
 			for(i=0; i<n_s; i++) {
 				/* Been there, done that: skip skip skip */
 				buffer += 4;
 				len -= 4;
+				if(len <= 0)	/* Make sure we don't overflow */
+					return -1;
 			}
 		}
 		if(gbit) {
 			if(!ybit) {
 				buffer++;
 				len--;
+				if(len == 0)	/* Make sure we don't overflow */
+					return -1;
 			}
 			uint8_t n_g = *buffer;
 			JANUS_LOG(LOG_HUGE, "There are %u frames in a GOF\n", n_g);
 			buffer++;
 			len--;
+			if(len == 0)	/* Make sure we don't overflow */
+				return -1;
 			if(n_g > 0) {
 				int i=0;
 				for(i=0; i<n_g; i++) {
@@ -917,9 +962,13 @@ int janus_vp9_parse_svc(char *buffer, int len, int *found,
 						/* Skip reference indices */
 						buffer += r;
 						len -= r;
+						if(len <= 0)	/* Make sure we don't overflow */
+							return -1;
 					}
 					buffer++;
 					len--;
+					if(len == 0)	/* Make sure we don't overflow */
+						return -1;
 				}
 			}
 		}
